@@ -598,7 +598,7 @@ zfs_setprop_error(libzfs_handle_t *hdl, zfs_prop_t prop, int err,
 			break;
 		}
 #endif
-		/* FALLTHROUGH */
+		fallthrough;
 	default:
 		(void) zfs_standard_error(hdl, err, errbuf);
 	}
